@@ -4,10 +4,12 @@ use warnings;
 use strict;
 require v5.10;
 use feature 'switch';
-use Carp 'croak';
 use Exporter 'import';
 our @EXPORT = qw{ dmap };
+our @EXPORT_OK = qw{ cut };
+use Carp 'croak';
 use Scalar::Util qw{ reftype refaddr };
+use Try::Tiny;
 
 =head1 NAME
 
@@ -15,11 +17,11 @@ Data::Dmap - just like map, but on deep data structures
 
 =head1 VERSION
 
-Version 0.04
+Version 0.05
 
 =cut
 
-our $VERSION = '0.04';
+our $VERSION = '0.05';
 
 =head1 SYNOPSIS
 
@@ -60,6 +62,8 @@ C<map>-like operation on deep data structures.
 
 =item C<dmap> - the dmap function that does deep in-place mapping
 
+=item C<cut> - a function for stopping recursion.
+
 =back
 
 =head1 SUBROUTINES/METHODS
@@ -127,7 +131,33 @@ data structures:
     #     }
     # }
     # (My own formatting above.)
-               
+
+=head2 C<cut>
+
+The C<cut> routine stops recursion at any point and returns any data as it is
+in place of the current node.
+
+=head3 Examples
+
+    use Data::Dmap 'cut';
+    use Data::Dump 'pp';
+    
+    my $deep = {
+        level => 1,
+        data  => {
+            level => 2,
+            data => {
+                level => 3
+            }
+        }
+    };
+    
+    pp dmap { cut('stop') if ref eq 'HASH' and $_->{level} == 2} $deep;
+
+    # Prints:
+    #
+    # { data => { data => "stop", level => 2 }, level => 1 }
+                       
                 
 =cut
 
@@ -157,49 +187,62 @@ sub _dmap {
         if(ref) {
             my $orig_ref = $_;
             if(not _has_cache($cache, $orig_ref)) {
-                my @mapped = $callback->($orig_ref);
-                foreach my $val (@mapped) {
-                    given(reftype $val) {
-                        when('HASH') {
-                            for(keys %$val) {
-                                my @res = _dmap($cache, $callback, $val->{$_});
-                                croak 'Multi value return in hash value assignment'
-                                    if @res > 1;
-                                if(@res) {
-                                    $val->{$_} = $res[0];
-                                } else {
-                                    delete $val->{$_};
-                                }
-                            }
-                            push @result, $val;
-                        }
-                        when('ARRAY') {
-                            my $i = 0;
-                            while($i <= $#$val) {
-                                if(exists $val->[$i]) {
-                                    # TODO Use splice to allow multi-value returns
-                                    my @res = _dmap($cache, $callback, $val->[$i]);
-                                    croak 'Multi value return in array single value assignment'
+                my $recurse = 1;
+                my @mapped;
+                try {
+                    @mapped = $callback->($orig_ref);
+                } catch {
+                    if(ref eq 'Data::Dmap::Cut') {
+                        $recurse = 0;
+                        @result = @$_;
+                    } else {
+                        die $_
+                    }
+                };
+                if($recurse) {
+                    foreach my $val (@mapped) {
+                        given(reftype $val) {
+                            when('HASH') {
+                                for(keys %$val) {
+                                    my @res = _dmap($cache, $callback, $val->{$_});
+                                    croak 'Multi value return in hash value assignment'
                                         if @res > 1;
                                     if(@res) {
-                                        $val->[$i] = $res[0];
+                                        $val->{$_} = $res[0];
                                     } else {
-                                        print splice @$val, $i, 1;
+                                        delete $val->{$_};
                                     }
                                 }
-                                $i++;
+                                push @result, $val;
+                            }
+                            when('ARRAY') {
+                                my $i = 0;
+                                while($i <= $#$val) {
+                                    if(exists $val->[$i]) {
+                                        # TODO Use splice to allow multi-value returns
+                                        my @res = _dmap($cache, $callback, $val->[$i]);
+                                        croak 'Multi value return in array single value assignment'
+                                            if @res > 1;
+                                        if(@res) {
+                                            $val->[$i] = $res[0];
+                                        } else {
+                                            print splice @$val, $i, 1;
+                                        }
+                                    }
+                                    $i++;
+                                }
+                                push @result, $val;
+                            }
+                            when('SCALAR') {
+                                my @res = _dmap($cache, $callback, $$val);
+                                croak 'Multi value return in single value assignment'
+                                    if @res > 1;
+                                $$val = $res[0] if @res;
+                                push @result, $val;
+                            }
+                            default {
+                                push @result, $val;
                             }
-                            push @result, $val;
-                        }
-                        when('SCALAR') {
-                            my @res = _dmap($cache, $callback, $$val);
-                            croak 'Multi value return in single value assignment'
-                                if @res > 1;
-                            $$val = $res[0] if @res;
-                            push @result, $val;
-                        }
-                        default {
-                            push @result, $val;
                         }
                     }
                 }
@@ -214,6 +257,8 @@ sub _dmap {
     } @_
 }
 
+sub cut { die bless [@_], 'Data::Dmap::Cut' }
+
 # Stub that inserts empty map cache
 sub dmap(&@) { _dmap({}, @_) }
 
@@ -268,8 +313,22 @@ L<http://search.cpan.org/dist/Data-Dmap/>
 =back
 
 
-=head1 ACKNOWLEDGEMENTS
+=head1 SEE ALSO
 
+L<Data::Rmap>, L<Data::Visitor>, L<Data::Transformer>, L<Data::Visitor>, L<Data::Walk>.
+
+=head1 TODO
+
+=over1
+
+=item Some kind of option making it possible to traverse objects with L<Class::MOP>
+metaclasses, so we can avoid breaking encapsulation.
+
+=item Options to provide more information about the current node to the
+callback handler, such as path, depth and data types. Should not affect
+performance if not used.
+
+=back
 
 =head1 LICENSE AND COPYRIGHT
 
